Index: gfortran.h
===================================================================
--- gfortran.h	(revision 127522)
+++ gfortran.h	(working copy)
@@ -1867,6 +1867,7 @@ typedef struct
   int flag_openmp;
   int flag_sign_zero;
   int flag_module_private;
+  int flag_recursive;
 
   int fpe;
 
Index: lang.opt
===================================================================
--- lang.opt	(revision 127522)
+++ lang.opt	(working copy)
@@ -218,7 +218,7 @@ Set default accessibility of module enti
 
 fopenmp
 Fortran
-Enable OpenMP
+Enable OpenMP (sets fmax-stack-var-size to be unlimited)
 
 fpack-derived
 Fortran
@@ -240,6 +240,10 @@ frecord-marker=8
 Fortran RejectNegative
 Use an 8-byte record marker for unformatted files
 
+frecursive
+Fortran
+Allocate local variables on the stack to allow indirect recursion
+
 frepack-arrays
 Fortran
 Copy array sections into a contiguous block on procedure entry
Index: invoke.texi
===================================================================
--- invoke.texi	(revision 127522)
+++ invoke.texi	(working copy)
@@ -156,7 +156,7 @@ and warnings}.
 -fsecond-underscore @gol
 -fbounds-check  -fmax-stack-var-size=@var{n} @gol
 -fpack-derived  -frepack-arrays  -fshort-enums  -fexternal-blas @gol
--fblas-matmul-limit=@var{n}}
+-fblas-matmul-limit=@var{n}} -frecursive
 @end table
 
 @menu
@@ -296,7 +296,8 @@ and @code{c$omp}, @code{*$omp} and @code
 @code{!$} conditional compilation sentinels in free form
 and @code{c$}, @code{*$} and @code{!$} sentinels in fixed form, 
 and when linking arranges for the OpenMP runtime library to be linked
-in.
+in.  The @option{-fopenmp} flag will override any setting of
+@option{-fmax-stack-var-size=}.
 
 @item -frange-check
 @opindex @code{frange-check}
@@ -865,7 +866,8 @@ substring references.
 @item -fmax-stack-var-size=@var{n}
 @opindex @code{fmax-stack-var-size}
 This option specifies the size in bytes of the largest array that will be put
-on the stack.
+on the stack.  The @option{-fopenmp} and @option{-frecursive} flags
+can override this setting.
 
 This option currently only affects local arrays declared with constant
 bounds, and may not apply to all character variables.
@@ -919,6 +921,12 @@ geometric mean of the dimensions of the 
 
 The default value for @var{n} is 30.
 
+@item -frecursive
+@opindex @code{frecursive}
+Allow indirect recursion by forcing all local arrays to be allocated
+on the stack.  This flag will override any value set with
+@option{fmax-stack-var-size=}.
+
 @end table
 
 @xref{Code Gen Options,,Options for Code Generation Conventions,
Index: options.c
===================================================================
--- options.c	(revision 127522)
+++ options.c	(working copy)
@@ -103,6 +103,7 @@ gfc_init_options (unsigned int argc ATTR
   gfc_option.flag_d_lines = -1;
   gfc_option.flag_openmp = 0;
   gfc_option.flag_sign_zero = 1;
+  gfc_option.flag_recursive = 0;
 
   gfc_option.fpe = 0;
 
@@ -294,6 +295,11 @@ gfc_post_options (const char **pfilename
   if (!gfc_option.flag_automatic)
     gfc_option.flag_max_stack_var_size = 0;
   
+  /* For OpenMP and indirect recusion, force local variables to
+     be allocated on the stack.  */
+  if (gfc_option.flag_openmp || gfc_option.flag_recursive)
+    gfc_option.flag_max_stack_var_size = -1;
+
   if (pedantic)
     { 
       gfc_option.warn_ampersand = 1;
@@ -698,6 +704,12 @@ gfc_handle_option (size_t scode, const c
 			 MAX_SUBRECORD_LENGTH);
 
       gfc_option.max_subrecord_length = value;
+      break;
+
+    case OPT_frecursive:
+      gfc_option.flag_recursive = 1;
+      break;
+
     }
 
   return result;
